# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.util import CLIError
from knack.log import get_logger
from msrestazure.tools import is_valid_resource_id, parse_resource_id
from azure.cli.core.commands import LongRunningOperation
from azure.cli.core.util import sdk_no_wait

from ._utils import (
    validate_managed_registry, get_registry_from_name_or_login_server, get_registry_by_name
)


logger = get_logger(__name__)

SOURCE_REGISTRY_NOT_FOUND = "The source container registry can not be found in the current subscription. " \
                            "Please provide a valid address and/or credentials."
IMPORT_NOT_SUPPORTED = "Imports are only supported for managed registries."
SOURCE_NOT_FOUND = "Source cannot be found. " \
                   "Please provide a valid image and source registry or a fully qualified source."
LOGIN_SERVER_NOT_VALID = "Login server of the registry is not valid " \
                         "because it is not a fully qualified domain name."
CREDENTIALS_INVALID = "Authentication failed. Please provide password."


def acr_import(cmd,  # pylint: disable=too-many-locals
               client,
               registry_name,
               source_image,
               source_registry=None,
               source_registry_username=None,
               source_registry_password=None,
               target_tags=None,
               resource_group_name=None,
               repository=None,
               force=False,
               no_wait=False):

    if source_registry_username and not source_registry_password:
        raise CLIError(CREDENTIALS_INVALID)

    _, resource_group_name = validate_managed_registry(
        cmd, registry_name, resource_group_name, IMPORT_NOT_SUPPORTED)

    ImportImageParameters, ImportSource, ImportMode = cmd.get_models(
        'ImportImageParameters', 'ImportSource', 'ImportMode')

    registry = None
    if source_registry:
        if is_valid_resource_id(source_registry):
            source = ImportSource(resource_id=source_registry, source_image=source_image)
        else:
            registry = get_registry_from_name_or_login_server(cmd.cli_ctx, source_registry, source_registry)
            if registry:
                # trim away redundant login server name, a common error
                prefix = registry.login_server + '/'
                if source_image.lower().startswith(prefix.lower()):
                    warning = ('The login server name of "%s" in the "--source" argument will be ignored as '
                               '"--registry" already supplies the same information')
                    logger.warning(warning, prefix[:-1])
                    source_image = source_image[len(prefix):]
                # For Azure container registry
                source = ImportSource(resource_id=registry.id, source_image=source_image)
            else:
                # For non-Azure container registry
                raise CLIError(SOURCE_REGISTRY_NOT_FOUND)

    else:
        registry_uri, source_image = _split_registry_and_image(source_image)
        if source_registry_password:
            ImportSourceCredentials = cmd.get_models('ImportSourceCredentials')
            source = ImportSource(registry_uri=registry_uri,
                                  source_image=source_image,
                                  credentials=ImportSourceCredentials(password=source_registry_password,
                                                                      username=source_registry_username))
        else:
            registry = get_registry_from_name_or_login_server(cmd.cli_ctx, registry_uri)
            if registry:
                # For Azure container registry
                source = ImportSource(resource_id=registry.id, source_image=source_image)
            else:
                # For non-Azure container registry
                source = ImportSource(registry_uri=registry_uri, source_image=source_image)

    if not target_tags and not repository:
        index = source_image.find("@")
        if index > 0:
            target_tags = [source_image[:index]]
        else:
            target_tags = [source_image]

    import_parameters = ImportImageParameters(source=source,
                                              target_tags=target_tags,
                                              untagged_target_repositories=repository,
                                              mode=ImportMode.force.value if force else ImportMode.no_force.value)

    try:
        if no_wait:
            logger.warning('Import has started. Due to no-wait option, failures will not be reflected.')
            return sdk_no_wait(no_wait, client.begin_import_image, resource_group_name,
                               registry_name, import_parameters)

        result_poller = client.begin_import_image(
            resource_group_name=resource_group_name,
            registry_name=registry_name,
            parameters=import_parameters)
        return LongRunningOperation(cmd.cli_ctx, 'Importing image...')(result_poller)

    except CLIError as e:
        _handle_import_exception(e, cmd, source_registry, source_image, registry)


def _handle_import_exception(e, cmd, source_registry, source_image, registry):
    from msrestazure.azure_exceptions import ClientException
    try:
        # if command fails, it might be because user specified registry twice in --source and --registry
        if source_registry:

            if not hasattr(registry, 'login_server'):
                if is_valid_resource_id(source_registry):
                    registry, _ = get_registry_by_name(cmd.cli_ctx, parse_resource_id(source_registry)["name"])
                else:
                    registry = get_registry_from_name_or_login_server(cmd.cli_ctx, source_registry, source_registry)

            if registry.login_server.lower() in source_image.lower():
                logger.warning("Import from source failed.\n\tsource image: '%s'\n"
                               "Attention: When source registry is specified with `--registry`, "
                               "`--source` is considered to be a source image name. "
                               "Do not prefix `--source` with the registry login server name.", "{}/{}"
                               .format(registry.login_server, source_image))
    except (ClientException, CLIError) as unexpected_ex:  # raise exception
        logger.debug("Unexpected exception: %s", unexpected_ex)

    raise e  # regardless re-raise the CLIError as this is an error from the service


def _split_registry_and_image(source_image):
    slash = source_image.find('/')
    if slash <= 0:
        raise CLIError(SOURCE_NOT_FOUND)

    registry_uri = source_image[:slash]
    if '.' not in registry_uri:
        raise CLIError(LOGIN_SERVER_NOT_VALID)

    source_image = source_image[slash + 1:]
    return registry_uri, source_image



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import threading
from knack.log import get_logger

logger = get_logger(__name__)


class TokenUpdater:
    """
    This class updates a given token_credential periodically using the provided callback function.
    It shows one way of making sure the credential does not become expired.
    """
    def __init__(self, token_credential, cli_ctx):
        self.token_credential = token_credential
        self.cli_ctx = cli_ctx

        # the timer needs to be protected, as later on it is possible that one thread is setting a new timer and
        # another thread is trying to cancel the timer
        self.lock = threading.Lock()
        self.timer_callback()

    def timer_callback(self):
        # call to get a new token and set a timer
        from azure.cli.core._profile import Profile
        from datetime import datetime
        # should give back token that is valid for at least 5 mins
        token = Profile(cli_ctx=self.cli_ctx).get_raw_token(
            resource="https://storage.azure.com", subscription=self.cli_ctx.data['subscription_id'])[0][2]
        try:
            self.token_credential.token = token['accessToken']
            expire = token['expiresOn']
            seconds_left = (datetime.strptime(expire, "%Y-%m-%d %H:%M:%S.%f") - datetime.now()).total_seconds()
        except KeyError:  # needed to deal with differing unserialized MSI token payload
            self.token_credential.token = token['access_token']
            expire = datetime.fromtimestamp(int(token['expires_on']))
            seconds_left = (expire - datetime.now()).total_seconds()

        if seconds_left < 180:
            logger.warning("Acquired token will expire on %s. Current time is %s.", expire, datetime.now())

        with self.lock:
            self.timer = threading.Timer(seconds_left - 180, self.timer_callback)
            self.timer.daemon = True
            self.timer.start()

    def cancel(self):
        # the timer needs to be canceled once the command has finished executing
        # if not the timer will keep going
        with self.lock:
            self.timer.cancel()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

def get_deserialized_and_headers(_0, deserialized, response_headers):
    return (deserialized, response_headers)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import importlib


def import_aaz_by_profile(module_name):
    return importlib.import_module(f"azure.cli.command_modules.network.aaz.2018_03_01_hybrid.{module_name}")



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from azure.cli.command_modules.acs.azuremonitormetrics.constants import RULES_API


def delete_rule(cmd, cluster_subscription, cluster_resource_group_name, default_rule_group_name):
    from azure.cli.core.util import send_raw_request
    default_rule_group_id = \
        "/subscriptions/{0}/resourceGroups/{1}/providers/Microsoft.AlertsManagement/prometheusRuleGroups/{2}".format(
            cluster_subscription,
            cluster_resource_group_name,
            default_rule_group_name
        )
    headers = ['User-Agent=azuremonitormetrics.delete_rule.' + default_rule_group_name]
    url = "{0}{1}?api-version={2}".format(
        cmd.cli_ctx.cloud.endpoints.resource_manager,
        default_rule_group_id,
        RULES_API
    )
    send_raw_request(cmd.cli_ctx, "DELETE", url, headers=headers)


def delete_rules(cmd, cluster_subscription, cluster_resource_group_name, cluster_name):
    delete_rule(
        cmd,
        cluster_subscription,
        cluster_resource_group_name,
        "NodeRecordingRulesRuleGroup-{0}".format(cluster_name)
    )
    delete_rule(
        cmd,
        cluster_subscription,
        cluster_resource_group_name,
        "KubernetesRecordingRulesRuleGroup-{0}".format(cluster_name)
    )
    delete_rule(
        cmd,
        cluster_subscription,
        cluster_resource_group_name,
        "NodeRecordingRulesRuleGroup-Win-{0}".format(cluster_name)
    )
    delete_rule(
        cmd,
        cluster_subscription,
        cluster_resource_group_name,
        "NodeAndKubernetesRecordingRulesRuleGroup-Win-{0}".format(cluster_name)
    )



# coding=utf-8
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.help_files import helps  # pylint: disable=unused-import
# pylint: disable=line-too-long, too-many-lines

helps['extension'] = """
type: group
short-summary: Manage and update CLI extensions.
"""

helps['extension add'] = """
type: command
short-summary: Add an extension.
long-summary: To learn about installing extensions automatically, visit https://aka.ms/AzExtDynamicInstall.
parameters:
  - name: --system
    type: string
    short-summary: Use a system directory for the extension.
    long-summary: Default path is azure-cli-extensions folder under the CLI running python environment lib path, configurable by environment variable AZURE_EXTENSION_SYS_DIR. On Windows, you may need to open your shell as Administrator to run with the right permission.
examples:
  - name: Add extension by name
    text: az extension add --name anextension
  - name: Add extension from URL
    text: az extension add --source https://contoso.com/anextension-0.0.1-py2.py3-none-any.whl
  - name: Add extension from local disk
    text: az extension add --source ~/anextension-0.0.1-py2.py3-none-any.whl
  - name: Add extension from local disk and use pip proxy for dependencies
    text: az extension add --source ~/anextension-0.0.1-py2.py3-none-any.whl --pip-proxy https://user:pass@proxy.server:8080
  - name: Add extension to system directory
    text: az extension add --name anextension --system
  - name: Add a specific version of extension
    text: az extension add --name anextension --version 1.0.0
  - name: Upgrade the extension if already installed
    text: az extension add --upgrade --name anextension
"""

helps['extension list'] = """
type: command
short-summary: List the installed extensions.
"""

helps['extension list-available'] = """
type: command
short-summary: List publicly available extensions.
examples:
  - name: List all publicly available extensions
    text: az extension list-available
  - name: List details on a particular extension
    text: az extension list-available --show-details --query anextension
"""

helps['extension remove'] = """
type: command
short-summary: Remove an extension.
examples:
  - name: Remove an extension. (autogenerated)
    text: az extension remove --name MyExtension
    crafted: true
"""

helps['extension show'] = """
type: command
short-summary: Show an extension.
examples:
  - name: Show an extension. (autogenerated)
    text: az extension show --name MyExtension
    crafted: true
"""

helps['extension update'] = """
type: command
short-summary: Update an extension.
examples:
  - name: Update an extension by name
    text: az extension update --name anextension
  - name: Update an extension by name and use pip proxy for dependencies
    text: az extension update --name anextension --pip-proxy https://user:pass@proxy.server:8080
"""

helps['extension list-versions'] = """
type: command
short-summary: List available versions for an extension.
examples:
  - name: List available versions for an extension
    text: az extension list-versions --name anextension
"""



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


DURATION_DAYS = 10675199
DURATION_MIN = 10085
DURATION_SECS = 4775807
DURATION_LIMIT = 10675198
SYSTEM = 'SystemAssigned'
USER = 'UserAssigned'
SYSTEMUSER = 'SystemAssigned, UserAssigned'



# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------
# pylint: disable=too-many-statements
# pylint: disable=too-many-locals
# pylint: disable=bad-continuation
# pylint: disable=line-too-long

from azure.cli.core.commands import CliCommandType
from ..generated._client_factory import (
    cf_kusto_operation,
    cf_kusto_pool,
    cf_kusto_pool,
    cf_kusto_pool_attached_database_configuration,
    cf_kusto_pool_database,
    cf_kusto_pool_data_connection,
    cf_kusto_pool_principal_assignment,
    cf_kusto_pool_database_principal_assignment,
)


synapse_kusto_pool_attached_database_configuration = CliCommandType(
    operations_tmpl='azure.mgmt.synapse.operations._kusto_pool_attached_database_configurations_operations#KustoPoolAttachedDatabaseConfigurationsOperations.{}',
    client_factory=cf_kusto_pool_attached_database_configuration,
)


synapse_kusto_pool_data_connection = CliCommandType(
    operations_tmpl=(
        'azure.mgmt.synapse.operations._kusto_pool_data_connections_operations#KustoPoolDataConnectionsOperations.{}'
    ),
    client_factory=cf_kusto_pool_data_connection,
)


synapse_kusto_pool_database = CliCommandType(
    operations_tmpl='azure.mgmt.synapse.operations._kusto_pool_databases_operations#KustoPoolDatabasesOperations.{}',
    client_factory=cf_kusto_pool_database,
)


synapse_kusto_pool_database_principal_assignment = CliCommandType(
    operations_tmpl='azure.mgmt.synapse.operations._kusto_pool_database_principal_assignments_operations#KustoPoolDatabasePrincipalAssignmentsOperations.{}',
    client_factory=cf_kusto_pool_database_principal_assignment,
)


synapse_kusto_pool = CliCommandType(
    operations_tmpl='azure.mgmt.synapse.operations._kusto_pool_operations#KustoPoolOperations.{}',
    client_factory=cf_kusto_pool,
)


synapse_kusto_pool = CliCommandType(
    operations_tmpl='azure.mgmt.synapse.operations._kusto_pools_operations#KustoPoolsOperations.{}',
    client_factory=cf_kusto_pool,
)


synapse_kusto_pool_principal_assignment = CliCommandType(
    operations_tmpl='azure.mgmt.synapse.operations._kusto_pool_principal_assignments_operations#KustoPoolPrincipalAssignmentsOperations.{}',
    client_factory=cf_kusto_pool_principal_assignment,
)


synapse_kusto_operation = CliCommandType(
    operations_tmpl='azure.mgmt.synapse.operations._kusto_operations_operations#KustoOperationsOperations.{}',
    client_factory=cf_kusto_operation,
)


def load_command_table(self, _):

    with self.command_group(
        'synapse kusto attached-database-configuration',
        synapse_kusto_pool_attached_database_configuration,
        client_factory=cf_kusto_pool_attached_database_configuration,
        is_experimental=True,
    ) as g:
        g.custom_command('list', 'synapse_kusto_attached_database_configuration_list')
        g.custom_show_command('show', 'synapse_kusto_attached_database_configuration_show')
        g.custom_command('create', 'synapse_kusto_attached_database_configuration_create', supports_no_wait=True)
        g.generic_update_command(
            'update',
            supports_no_wait=True,
            custom_func_name='synapse_kusto_attached_database_configuration_update',
            setter_name='begin_create_or_update',
        )
        g.custom_command(
            'delete', 'synapse_kusto_attached_database_configuration_delete', supports_no_wait=True, confirmation=True
        )
        g.custom_wait_command('wait', 'synapse_kusto_attached_database_configuration_show')

    with self.command_group(
        'synapse kusto data-connection',
        synapse_kusto_pool_data_connection,
        client_factory=cf_kusto_pool_data_connection,
        is_experimental=True,
    ) as g:
        g.custom_command('list', 'synapse_kusto_data_connection_list')
        g.custom_show_command('show', 'synapse_kusto_data_connection_show')
        g.custom_command('event-grid create', 'synapse_kusto_data_connection_event_grid_create', supports_no_wait=True)
        g.custom_command('event-hub create', 'synapse_kusto_data_connection_event_hub_create', supports_no_wait=True)
        g.custom_command('iot-hub create', 'synapse_kusto_data_connection_iot_hub_create', supports_no_wait=True)
        g.custom_command('event-grid update', 'synapse_kusto_data_connection_event_grid_update', supports_no_wait=True)
        g.custom_command('event-hub update', 'synapse_kusto_data_connection_event_hub_update', supports_no_wait=True)
        g.custom_command('iot-hub update', 'synapse_kusto_data_connection_iot_hub_update', supports_no_wait=True)
        g.custom_command('delete', 'synapse_kusto_data_connection_delete', supports_no_wait=True, confirmation=True)
        g.custom_wait_command('wait', 'synapse_kusto_data_connection_show')

    with self.command_group(
        'synapse kusto database',
        synapse_kusto_pool_database,
        client_factory=cf_kusto_pool_database,
        is_experimental=True,
    ) as g:
        g.custom_command('list', 'synapse_kusto_database_list')
        g.custom_show_command('show', 'synapse_kusto_database_show')
        g.custom_command('create', 'synapse_kusto_database_create', supports_no_wait=True)
        g.custom_command('update', 'synapse_kusto_database_update', supports_no_wait=True)
        g.custom_command('delete', 'synapse_kusto_database_delete', supports_no_wait=True, confirmation=True)
        g.custom_wait_command('wait', 'synapse_kusto_database_show')

    with self.command_group(
        'synapse kusto database-principal-assignment',
        synapse_kusto_pool_database_principal_assignment,
        client_factory=cf_kusto_pool_database_principal_assignment,
        is_experimental=True,
    ) as g:
        g.custom_command('list', 'synapse_kusto_database_principal_assignment_list')
        g.custom_show_command('show', 'synapse_kusto_database_principal_assignment_show')
        g.custom_command('create', 'synapse_kusto_database_principal_assignment_create', supports_no_wait=True)
        g.generic_update_command(
            'update',
            supports_no_wait=True,
            custom_func_name='synapse_kusto_database_principal_assignment_update',
            setter_name='begin_create_or_update',
        )
        g.custom_command(
            'delete', 'synapse_kusto_database_principal_assignment_delete', supports_no_wait=True, confirmation=True
        )
        g.custom_wait_command('wait', 'synapse_kusto_database_principal_assignment_show')

    with self.command_group(
        'synapse kusto pool', synapse_kusto_pool, client_factory=cf_kusto_pool, is_experimental=True
    ) as g:
        g.custom_command('list-sku', 'synapse_kusto_pool_list_sku')

    with self.command_group(
        'synapse kusto pool', synapse_kusto_pool, client_factory=cf_kusto_pool, is_experimental=True
    ) as g:
        g.custom_command('list', 'synapse_kusto_pool_list')
        g.custom_show_command('show', 'synapse_kusto_pool_show')
        g.custom_command('delete', 'synapse_kusto_pool_delete', supports_no_wait=True, confirmation=True)
        g.custom_command('list-follower-database', 'synapse_kusto_pool_list_follower_database')
        g.custom_command('list-language-extension', 'synapse_kusto_pool_list_language_extension')
        g.custom_command('list-sku', 'synapse_kusto_pool_list_sku')
        g.custom_command('start', 'synapse_kusto_pool_start', supports_no_wait=True)
        g.custom_command('stop', 'synapse_kusto_pool_stop', supports_no_wait=True)
        g.custom_wait_command('wait', 'synapse_kusto_pool_show')

    with self.command_group(
        'synapse kusto pool-principal-assignment',
        synapse_kusto_pool_principal_assignment,
        client_factory=cf_kusto_pool_principal_assignment,
        is_experimental=True,
    ) as g:
        g.custom_command('list', 'synapse_kusto_pool_principal_assignment_list')
        g.custom_show_command('show', 'synapse_kusto_pool_principal_assignment_show')
        g.custom_command('create', 'synapse_kusto_pool_principal_assignment_create', supports_no_wait=True)
        g.generic_update_command(
            'update',
            supports_no_wait=True,
            custom_func_name='synapse_kusto_pool_principal_assignment_update',
            setter_name='begin_create_or_update',
        )
        g.custom_command(
            'delete', 'synapse_kusto_pool_principal_assignment_delete', supports_no_wait=True, confirmation=True
        )
        g.custom_wait_command('wait', 'synapse_kusto_pool_principal_assignment_show')

    with self.command_group(
        'synapse kusto-operation', synapse_kusto_operation, client_factory=cf_kusto_operation, is_experimental=True
    ) as g:
        g.custom_command('list', 'synapse_kusto_operation_list')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=line-too-long, raise-missing-from
from collections import OrderedDict
from azure.cli.core.util import CLIError


def table_transform_output(result):
    table_result = []
    for key in ('host', 'username', 'password', 'location', 'skuname', 'resource group', 'id', 'version', 'connection string'):
        entry = OrderedDict()
        entry['Property'] = key
        entry['Value'] = result[key]
        table_result.append(entry)

    return table_result


def table_transform_output_list_servers(result):

    table_result = []

    if not result:
        return table_result

    for key in result:
        new_entry = OrderedDict()
        new_entry['Name'] = key['name']
        new_entry['Resource Group'] = key['resourceGroup']
        new_entry['Location'] = key['location']
        new_entry['Version'] = key['version']
        new_entry['Storage Size(GiB)'] = int(key['storage']['storageSizeGb'])
        new_entry['Tier'] = key['sku']['tier']
        new_entry['SKU'] = key['sku']['name']

        if 'flexibleServers' in result[0]['id']:
            new_entry['State'] = key['state']
            new_entry['HA State'] = key['highAvailability']['state']
            new_entry['Availability zone'] = key['availabilityZone']

        table_result.append(new_entry)

    return table_result


def mysql_table_transform_output_list_skus(result):
    table_result = []
    if len(result) > 1:
        skus_tiers = result[0]["supportedFlexibleServerEditions"]
        for skus in skus_tiers:
            tier_name = skus["name"]
            try:
                keys = skus["supportedServerVersions"][0]["supportedSkus"]
                for key in keys:
                    new_entry = OrderedDict()
                    new_entry['SKU'] = key['name']
                    new_entry['Tier'] = tier_name
                    new_entry['vCore'] = key['vCores']
                    new_entry['Memory'] = str(int(key['supportedMemoryPerVCoreMb']) * int(key['vCores']) // 1024) + " GiB"
                    new_entry['Max Disk IOPS'] = key['supportedIops']
                    table_result.append(new_entry)
            except:
                raise CLIError("There is no sku available for this location.")

    return table_result


def table_transform_output_parameters(result):

    table_result = []

    if not result:
        return table_result

    for key in result:
        new_entry = OrderedDict()
        new_entry['Name'] = key['name']
        new_entry['DataType'] = key['dataType']
        new_entry['DefaultValue'] = key['defaultValue']
        new_entry['Source'] = key['source']
        new_entry['AllowedValues'] = key['allowedValues']

        table_result.append(new_entry)

    return table_result


def transform_backup(result):
    if result.type == 'Microsoft.DBforMySQL/flexibleServers/backups' and len(result.name) > 62:
        result.name = result.name[62:]

    if hasattr(result, 'system_data'):
        delattr(result, 'system_data')

    return result


def transform_backups_list(results):
    return [transform_backup(result) for result in results]
